import { watch, shallowRef, Ref } from "vue";

/**
 * Custom debounce composable for Vue.
 *
 * @param value - The reactive value to debounce.
 * @param delay - The debounce delay in milliseconds (default is 500ms).
 * @returns A debounced reactive value.
 */
export function useDebounce<T>(value: Ref<T>, delay: number = 500): Ref<T> {
  const debouncedValue = shallowRef(value.value); // Use shallowRef to avoid deep unwrapping

  let timer: ReturnType<typeof setTimeout> | null = null;

  watch(
    value,
    (newValue) => {
      if (timer) clearTimeout(timer);

      timer = setTimeout(() => {
        debouncedValue.value = newValue;
      }, delay);
    },
    { immediate: true } // Apply watch immediately for initialization
  );

  return debouncedValue as Ref<T>;
}