import { ITHit } from "webdav.client";
import { SenderType, UploadItemRow } from "./uploadItemRow";
import { useGridStore } from "../stores/gridStore";
import { useUploadStore } from "../stores/uploadStore";
import { UrlResolveService } from "../services/urlResolveService";
import { WebDavService } from "../services/webDavService";
import { WebDavError } from "./webDavError";
import { RewriteItemsData } from "./rewriteItemsData";
import i18nConfig from "../i18n/i18n";

export interface UploadItemRowHandlers {
  onProgress: Function;
  onStateChanged: Function;
  onBeforeUploadStarted: Function;
  onUploadError: Function;
}

export function createUploadItemHandlers(row: UploadItemRow) {
  const gridStore = useGridStore();
  const uploadStore = useUploadStore();
  const i18n = i18nConfig.global;

  const handlers: UploadItemRowHandlers = {
    onProgress: (e: ITHit.WebDAV.Client.Upload.Events.ProgressChanged) => {
      row.currentState = (e.Sender as SenderType).GetState();
    },
    onStateChanged: (e: ITHit.WebDAV.Client.Upload.Events.StateChanged) => {
      row.currentState = e.NewState;
      if (e.NewState === ITHit.WebDAV.Client.Upload.State.Completed) {
        gridStore.refresh();
      }
    },
    onBeforeUploadStarted: (e: ITHit.WebDAV.Client.Upload.Events.BeforeUploadStarted) => {
      const oItem = e.Sender as ITHit.WebDAV.Client.Upload.UploadItem;
      const sHref = UrlResolveService.encodeUri(oItem.GetUrl());
      if (
        oItem.GetOverwrite() ||
        oItem.IsFolder() ||
        (oItem.CustomData as { FileExistanceVerified: boolean }).FileExistanceVerified
      ) {
        e.Upload();
        return;
      }
      WebDavService.openItem(sHref, (oAsyncResult) => {
        if (!oAsyncResult.IsSuccess && oAsyncResult.Status.Code === 404) {
          e.Upload();
          return;
        }

        if (!oAsyncResult.IsSuccess) {
          gridStore.setError(new WebDavError(i18n.t("phrases.errors.failedCheckExistsErrorMessage"), oAsyncResult.Error));
          (e.Sender as SenderType).SetFailed(oAsyncResult.Error);
          return;
        }

        const rewriteData = new RewriteItemsData(
          /* A user selected to overwrite existing file. */
          function () {
            // Do not delete item if upload canceled (it existed before the upload).
            (e.Sender as SenderType).SetDeleteOnCancel(false);

            // The item will be overwritten if it exists on the server.
            (e.Sender as SenderType).SetOverwrite(true);

            // All async requests completed - start upload.
            e.Upload();
          },
          /* A user selected to skip existing files. */
          function () { },
          oItem.GetRelativePath()
        );
        uploadStore.setRewriteItemsData(rewriteData);
      });
    },
    onUploadError: (e: ITHit.WebDAV.Client.Upload.Events.UploadError) => {
      if (row.maxRetry <= row.currentRetry) {
        e.Skip();
        return;
      }

      // Retry upload.
      const retryTime = new Date().getTime() + row.retryDelay * 1000;
      const retryTimerId = setInterval(() => {
        const timeLeft = retryTime - new Date().getTime();
        if (timeLeft > 0) {
          row.setRetryMessage(timeLeft);
          return;
        }
        clearInterval(retryTimerId);
        row.currentRetry++;
        row.removeRetryMessage();

        // Request number of bytes successfully saved on the server
        // and retry upload from next byte.
        e.Retry();
      }, 1000);

      row.cancelRetryCallback = function () {
        clearInterval(retryTimerId);
        row.removeRetryMessage();
      };
    }
  };

  return handlers;
}