import { ITHit } from "webdav.client";
import { CommonService } from "../services/commonService";
import { UploadItemRowHandlers } from "./uploadItemHandlers";

export interface SenderType {
  GetState(): string;
  SetFailed(
    error:
      | ITHit.WebDAV.Client.Exceptions.WebDavException
      | ITHit.WebDAV.Client.Error
      | null
  ): void;
  SetDeleteOnCancel(value: boolean): void;
  SetOverwrite(value: boolean): void;
  Upload(): void;
}

export class UploadItemRow {
  public currentState: string = "";
  public retryMessage: string = "";
  public uploadItem: ITHit.WebDAV.Client.Upload.UploadItem;
  public handlers: UploadItemRowHandlers | null;

  public maxRetry: number = 10;
  public currentRetry: number = 0;
  public retryDelay: number = 10;
  public cancelRetryCallback: (() => void) | null = null;

  constructor(uploadItem: ITHit.WebDAV.Client.Upload.UploadItem) {
    this.uploadItem = uploadItem;
    this.currentState = this.uploadItem.GetState();
    this.handlers = null;
  }

  addHandlers(handlers: UploadItemRowHandlers) {
    this.handlers = handlers;
    this.uploadItem.AddListener(
      "OnProgressChanged",
      (e: ITHit.WebDAV.Client.Upload.Events.ProgressChanged) =>
        handlers.onProgress(e)
    );
    this.uploadItem.AddListener(
      "OnStateChanged",
      (e: ITHit.WebDAV.Client.Upload.Events.StateChanged) =>
        handlers.onStateChanged(e)
    );
    this.uploadItem.AddListener(
      "OnBeforeUploadStarted",
      (e: ITHit.WebDAV.Client.Upload.Events.BeforeUploadStarted) =>
        handlers.onBeforeUploadStarted(e)
    );
    this.uploadItem.AddListener(
      "OnUploadError",
      (e: ITHit.WebDAV.Client.Upload.Events.UploadError) =>
        handlers.onUploadError(e)
    );
  }

  destroy() {
    if (this.handlers !== null) {
      this.uploadItem.RemoveListener(
        "OnProgressChanged",
        this.handlers.onProgress
      );
      this.uploadItem.RemoveListener(
        "OnStateChanged",
        this.handlers.onStateChanged
      );
      this.uploadItem.RemoveListener(
        "OnBeforeUploadStarted",
        this.handlers.onBeforeUploadStarted
      );
      this.uploadItem.RemoveListener(
        "OnUploadError",
        this.handlers.onUploadError
      );
    }
  }

  cancelClickHandler() {
    this.cancelRetry();
    this.uploadItem.CancelAsync();
  }
  pauseClickHandler(fCallback?: () => void) {
    this.cancelRetry();
    this.uploadItem.PauseAsync(fCallback);
  }
  playClickHandler(fCallback?: () => void) {
    this.currentRetry = 0;
    this.uploadItem.StartAsync(fCallback);
  }

  setRetryMessage(timeLeft: number) {
    const sRetryMessageFormat = "Retry in: {0}";
    this.retryMessage = CommonService.pasteFormat(
      sRetryMessageFormat,
      CommonService.timeSpan(Math.ceil(timeLeft / 1000))
    );
  }

  removeRetryMessage() {
    this.retryMessage = "";
  }

  cancelRetry() {
    if (this.cancelRetryCallback) this.cancelRetryCallback.call(this);
  }
}
