import { WebDavSettings } from "../models/webDavSettings";
import { UrlResolveService } from "./urlResolveService";
import { useGridStore } from "../stores/gridStore";

export function WebSocketConnect() {
  const gridStore = useGridStore();

  let socketSource: WebSocket | undefined;

  if (WebDavSettings.WebSocketPath) {
    socketSource = new WebSocket(WebDavSettings.WebSocketPath);
  } else {
    let host = window.location.host;

    if (WebDavSettings.WebsiteRootUrl) {
      const oAppPath = new URL(WebDavSettings.WebsiteRootUrl);
      host = oAppPath.host;
    }

    if (window.location.protocol === "https:") {
      socketSource = new WebSocket("wss://" + host);
    } else {
      socketSource = new WebSocket("ws://" + host);
    }
  }

  socketSource.addEventListener(
    "message",
    function (e: MessageEvent) {
      const notifyObject = JSON.parse(e.data);
      // Removing domain and trailing slash.
      const currentLocation = window.location.pathname.replace(/^\/|\/$/g, "");
      const itemPath = notifyObject.ItemPath;
      // Checking message type after receiving.
      if (
        notifyObject.EventType === "updated" ||
        notifyObject.EventType === "locked" ||
        notifyObject.EventType === "unlocked"
      ) {
        if (
          itemPath.substring(0, itemPath.lastIndexOf("/")).toUpperCase() ===
          currentLocation.toUpperCase()
        ) {
          gridStore.setItem(
            new URL(UrlResolveService.getRootUrl()).origin + "/" + itemPath
          );
        }
      } else if (notifyObject.EventType === "created") {
        // Refresh folder structure if any item in this folder is updated or new item is created.
        if (
          itemPath.substring(0, itemPath.lastIndexOf("/")).toUpperCase() ===
          currentLocation.toUpperCase()
        ) {
          gridStore.refreshFolder();
        }
      } else if (notifyObject.EventType === "moved") {
        // Refresh folder structure if file or folder is moved.
        if (
          itemPath.substring(0, itemPath.lastIndexOf("/")).toUpperCase() ===
          currentLocation.toUpperCase() ||
          notifyObject.TargetPath.substring(
            0,
            notifyObject.TargetPath.lastIndexOf("/")
          ).toUpperCase() === currentLocation.toUpperCase()
        ) {
          gridStore.refreshFolder();
        }
      } else if (notifyObject.EventType === "deleted") {
        if (
          itemPath.substring(0, itemPath.lastIndexOf("/")).toUpperCase() ===
          currentLocation.toUpperCase()
        ) {
          // Refresh folder structure if any item in this folder is deleted.
          gridStore.refreshFolder();
        } else if (
          currentLocation.toUpperCase().indexOf(itemPath.toUpperCase()) === 0
        ) {
          // Redirect client to the root folder if current path is being deleted.
          const originPath = window.location.origin + "/";
          window.history.pushState({ Url: originPath }, "", originPath);
          gridStore.refresh(originPath);
        }
      }
    },
    false
  );
  socketSource.addEventListener("error", function (err: Event) {
    console.error(
      "Socket encountered error: ",
      (err as ErrorEvent).message,
      "Closing socket"
    );
    socketSource?.close();
  });

  socketSource.addEventListener("close", function (_e: CloseEvent) {
    setTimeout(function () {
      WebSocketConnect();
    }, 5000);
  });
}
