import { defineStore } from "pinia";
import { UrlResolveService } from "../services/urlResolveService";
import { ITHit } from "webdav.client";
import { StoredType } from "../models/storedType";
import { WebDavError } from "../models/webDavError";
import { QueryParams } from "../models/queryParams";
import i18nConfig from "../i18n/i18n";
import { CommonService } from "../services/commonService";
import { WebDavService } from "../services/webDavService";
import { WebDavSettings } from "../models/webDavSettings";
import { SearchParams } from "../models/searchParams";
const i18n = i18nConfig.global;

export interface CurrentFolderResult {
  page: ITHit.WebDAV.Client.HierarchyItem[];
  totalItems: number;
}

export interface CurrentFolderResponse {
  folder: ITHit.WebDAV.Client.Folder;
  result: CurrentFolderResult;
  withSearch: boolean;
}

export interface GridState {
  sortAscending: boolean;
  sortColumn: string;
  searchQuery: string;
  searchMode: boolean;
  currentFolder: ITHit.WebDAV.Client.Folder | null;
  items: ITHit.WebDAV.Client.HierarchyItem[];
  searchedItems: ITHit.WebDAV.Client.HierarchyItem[];
  storedItems: ITHit.WebDAV.Client.HierarchyItem[];
  selectedIndexes: number[];
  loading: boolean;
  loadingWithSceleton: boolean;
  error: WebDavError | null;
  currentPage: number;
  pageSize: number;
  countPages: number;
  currentUrl: string;
  storedType: StoredType;
  optionsInfo: ITHit.WebDAV.Client.OptionsInfo | null;
  optionsInfoLoading: boolean;
  protocolModalDisplayed: boolean;
  countPagesLoaded: number;
  loadMore: boolean;
  totalItemsCount: number;
}

export const state: GridState = {
  sortAscending: true,
  sortColumn: "displayname",
  searchQuery: "",
  searchMode: false,
  currentFolder: null,
  items: [],
  searchedItems: [],
  storedItems: [],
  selectedIndexes: [],
  loading: true,
  loadingWithSceleton: false,
  error: null,
  pageSize: 20,
  currentPage: 1,
  countPages: 0,
  currentUrl: UrlResolveService.getRootUrl(),
  storedType: StoredType.Copy,
  optionsInfo: null,
  optionsInfoLoading: true,
  protocolModalDisplayed: false,
  countPagesLoaded: 0,
  loadMore: false,
  totalItemsCount: 0,
};

export const initialState = { ...state };

export const useGridStore = defineStore("grid", {
  state: (): GridState => state,
  getters: {
    isAllSelected: (state) => {
      return state.selectedIndexes.length === state.items.length;
    },
    selectedItems: (state) => {
      return state.items.filter((_item, index) =>
        state.selectedIndexes.includes(index)
      );
    },
  },
  actions: {
    async fetchFolderItems(
      currentUrl: string,
      pageSize: number,
      sortColumn: string,
      sortAscending: boolean,
      page: number,
      withSearch: boolean,
      searchQuery?: string
    ): Promise<CurrentFolderResponse> {
      try {
        const response = await WebDavService.getCurrentFolder(currentUrl);
        const folder = response.Result as ITHit.WebDAV.Client.Folder;

        let itemsResponse;
        if (withSearch && searchQuery) {
          itemsResponse = await WebDavService.getItemsByQuery(
            folder,
            page,
            pageSize,
            searchQuery
          );
        } else {
          itemsResponse = await WebDavService.getItems(
            folder,
            sortColumn,
            sortAscending,
            page,
            pageSize
          );
        }

        return {
          folder,
          result: {
            page: itemsResponse.Result.Page,
            totalItems: itemsResponse.Result.TotalItems,
          },
          withSearch,
        };
      } catch (err) {
        throw new WebDavError("Error fetching folder items", err);
      }
    },

    async setCurrentFolder(withSearch: boolean) {
      this.loading = true;
      try {
        const response = await this.fetchFolderItems(
          this.currentUrl,
          this.pageSize,
          this.sortColumn,
          this.sortAscending,
          this.currentPage,
          withSearch,
          this.searchQuery
        );
        this.items = response.result.page;
        this.countPages = Math.ceil(response.result.totalItems / this.pageSize);
        this.totalItemsCount = response.result.totalItems;
        this.countPagesLoaded = 1;
        this.currentFolder = response.folder;
        this.searchMode = response.withSearch;
      } catch (err) {
        this.error =
          err instanceof WebDavError ? err : new WebDavError("", err);
      } finally {
        this.loading = false;
      }
    },

    async refreshFolder() {
      try {
        const refreshCountItems =
          this.pageSize < this.items.length ? this.items.length : this.pageSize;
        const response = await this.fetchFolderItems(
          this.currentUrl,
          refreshCountItems,
          this.sortColumn,
          this.sortAscending,
          1,
          this.searchMode,
          this.searchQuery
        );
        this.items = response.result.page;
        this.countPages = Math.ceil(response.result.totalItems / this.pageSize);
        this.totalItemsCount = response.result.totalItems;
        this.selectedIndexes = [];
      } catch (err) {
        this.error =
          err instanceof WebDavError ? err : new WebDavError("", err);
      }
    },

    async appendNewPage() {
      try {
        const response = await this.fetchFolderItems(
          this.currentUrl,
          this.pageSize,
          this.sortColumn,
          this.sortAscending,
          this.countPagesLoaded + 1,
          false
        );
        this.items = [...this.items, ...response.result.page];
        this.countPages = Math.ceil(response.result.totalItems / this.pageSize);
        this.totalItemsCount = response.result.totalItems;
        this.countPagesLoaded += 1;
      } catch (err) {
        this.error =
          err instanceof WebDavError ? err : new WebDavError("", err);
      }
    },

    async setItem(itemPath: string) {
      try {
        const item = this.items.find(
          (p) =>
            UrlResolveService.decodeUri(p.Href) ===
            UrlResolveService.decodeUri(itemPath)
        );
        if (item && item.Href) {
          const response = await WebDavService.getItem(item.Href);
          if (response.IsSuccess) {
            const updatedItem =
              response.Result as ITHit.WebDAV.Client.HierarchyItem;
            const index = this.items.findIndex(
              (p) => p.DisplayName === updatedItem.DisplayName
            );
            if (index >= 0) {
              this.items[index] =
                updatedItem as ITHit.WebDAV.Client.HierarchyItem;
            }
          }
        }
      } catch (err) {
        this.error =
          err instanceof WebDavError ? err : new WebDavError("", err);
      }
    },

    async setOptionsInfo() {
      try {
        if (this.currentFolder) {
          const response = await WebDavService.getSupportedFeatures(
            this.currentFolder
          );
          this.optionsInfo = response.Result;
        }
      } catch (err) {
        this.error =
          err instanceof WebDavError ? err : new WebDavError("", err);
      }
    },

    async setSearchedItems(searchParams: SearchParams) {
      try {
        if (this.currentFolder) {
          const response = await WebDavService.getItemsByQuery(
            this.currentFolder,
            searchParams.pageNumber,
            searchParams.pageSize,
            searchParams.query
          );
          this.searchedItems = response.Result.Page;
        }
      } catch (err) {
        this.error = new WebDavError("Search error", err);
      }
    },
    setSortColumn(sortColumn: string) {
      this.sortColumn = sortColumn;
    },
    setSortAscending(sortAscending: boolean) {
      this.sortAscending = sortAscending;
    },
    setCurrentPage(currentPage: number) {
      this.currentPage = currentPage;
    },
    setSearchQuery(searchQuery: string) {
      this.searchQuery = searchQuery;
      this.searchMode = !!searchQuery;
    },
    setSearchedItem(item: ITHit.WebDAV.Client.HierarchyItem) {
      this.items = [item];
      this.currentPage = 1;
      this.countPages = 1;
      this.searchMode = true;
    },
    setLoadingWithSceleton(loading: boolean) {
      this.loadingWithSceleton = loading;
    },
    addSelectedItem(index: number) {
      this.selectedIndexes.push(index);
    },
    removeSelectedItem(index: number) {
      this.selectedIndexes = this.selectedIndexes.filter(
        (selectedIndex) => selectedIndex !== index
      );
    },
    clearSelectedItems() {
      this.selectedIndexes = [];
    },
    addAllSelectedItems() {
      if (this.items.length > 0) {
        this.selectedIndexes = Array.from(
          { length: this.items.length },
          (_, k) => k
        );
      } else {
        this.selectedIndexes = [];
      }
    },
    storeSelectedItems(storedType: StoredType) {
      this.storedItems = this.selectedIndexes.map((i) => this.items[i]);
      this.storedType = storedType;
    },
    clearStoredItems() {
      this.storedItems = [];
    },
    setCurrentUrl(url: string) {
      this.currentUrl = url;
    },
    clearItems() {
      this.items = [];
    },
    showProtocolModal() {
      this.protocolModalDisplayed = true;
    },
    hideProtocolModal() {
      this.protocolModalDisplayed = false;
    },
    setError(error: WebDavError) {
      this.error = error;
    },
    clearError() {
      this.error = null;
    },
    clearSearchedItems() {
      this.searchedItems = [];
    },
    pasteStoredItems() {
      switch (this.storedType) {
        case StoredType.Cut:
          this._moveItems();
          break;
        case StoredType.Copy:
          this._copyItems();
          break;
      }
      this.clearStoredItems();
    },
    deleteSelectedItems() {
      this.selectedItems.forEach((item) => {
        item.DeleteAsync(null, (data) => {
          if (data.IsSuccess) {
            this.refresh();
          } else {
            const error = new WebDavError(
              i18n.t("phrases.errors.deletetemsErrorMessage"),
              data.Error
            );
            this.setError(error);
          }
        });
      });
    },
    renameSelectedItem(name: string) {
      if (this.selectedItems.length === 1 && this.currentFolder !== null) {
        this.selectedItems[0].MoveToAsync(
          this.currentFolder,
          name,
          false,
          [],
          (data) => {
            if (data.IsSuccess) {
              this.refresh();
            } else {
              let error;
              if (
                data.Error instanceof
                ITHit.WebDAV.Client.Exceptions.LockedException
              ) {
                error = new WebDavError(
                  i18n.t("phrases.errors.renameItemLockedErrorMessage"),
                  data.Error
                );
              } else {
                error = new WebDavError(
                  i18n.t("phrases.errors.renameItemErrorMessage"),
                  data.Error
                );
              }
              this.setError(error);
            }
          }
        );
      }
    },
    printSelectedItems() {
      const filesUrls: string[] = [];
      this.selectedItems.forEach((item) => {
        if (!WebDavService.isFolder(item)) {
          filesUrls.push(item.Href);
        }
      });

      ITHit.WebDAV.Client.DocManager.DavProtocolEditDocument(
        filesUrls,
        UrlResolveService.getOrigin(),
        () => {
          this.showProtocolModal();
        },
        null,
        WebDavSettings.EditDocAuth.SearchIn,
        WebDavSettings.EditDocAuth.CookieNames,
        WebDavSettings.EditDocAuth.LoginUrl,
        "Print"
      );
    },
    refresh(
      url?: string | null,
      queryParams?: QueryParams | null,
      showSceleton?: boolean
    ) {
      let curSearchMode = this.searchMode;
      if (url) {
        this.setCurrentUrl(url);
      }
      this.setCurrentPage(1);
      if (queryParams) {
        this._setHash(queryParams);
        if (queryParams.search) {
          curSearchMode = true;
        } else {
          curSearchMode = false;
        }
      } else {
        if (!this.searchQuery && this.searchMode) {
          curSearchMode = false;
        }
      }

      if (curSearchMode !== this.searchMode) {
        this.setCurrentPage(1);
      }
      this.clearSelectedItems();
      this.setLoadingWithSceleton(!!showSceleton);
      this.setCurrentFolder(curSearchMode).then(() => {
        this.setOptionsInfo();
        const hash = this._getHash();
        if (hash) {
          window.history.pushState({}, "", hash);
        } else {
          window.history.pushState({}, "", window.location.pathname);
        }
      });
    },
    _moveItems() {
      this.storedItems.forEach((item) => {
        if (this.currentFolder !== null)
          item.MoveToAsync(
            this.currentFolder,
            item.DisplayName,
            false,
            [],
            (data) => {
              if (data.IsSuccess) {
                this.refresh();
              } else {
                let error;
                if (
                  data.Error instanceof
                  ITHit.WebDAV.Client.Exceptions.ForbiddenException
                ) {
                  error = new WebDavError(
                    i18n.t("phrases.errors.cutItemsSameNameErrorMessage"),
                    data.Error
                  );
                } else if (
                  data.Error instanceof
                  ITHit.WebDAV.Client.Exceptions.LockedException
                ) {
                  error = new WebDavError(
                    i18n.t("phrases.errors.cutItemsLockedErrorMessage"),
                    data.Error
                  );
                } else {
                  error = new WebDavError(
                    i18n.t("phrases.errors.cutItemsErrorMessage"),
                    data.Error
                  );
                }
                this.setError(error);
              }
            }
          );
      });
    },
    _copyItems() {
      this.storedItems.forEach((item) => {
        const itemCopyName = CommonService.getCopySuffix(
          item.DisplayName,
          false
        );
        if (this.currentFolder !== null) {
          this._copyItem(item, itemCopyName, this.currentFolder);
        }
      });
    },
    _copyItem(
      item: ITHit.WebDAV.Client.HierarchyItem,
      copyName: string,
      folder: ITHit.WebDAV.Client.Folder
    ) {
      item.CopyToAsync(folder, copyName, false, false, [], (data) => {
        if (data.IsSuccess) {
          this.refresh();
        } else {
          if (
            data.Error instanceof
              ITHit.WebDAV.Client.Exceptions.PreconditionFailedException ||
            data.Error instanceof
              ITHit.WebDAV.Client.Exceptions.ForbiddenException
          ) {
            const newCopyName = CommonService.getCopySuffix(copyName, true);
            this._copyItem(item, newCopyName, folder);
          } else {
            const error = new WebDavError(
              i18n.t("phrases.errors.copyItemsErrorMessage"),
              data.Error
            );
            this.setError(error);
          }
        }
      });
    },
    _getHash() {
      const hashArray = [];
      if (this.currentPage !== initialState.currentPage) {
        hashArray.push(`page=${this.currentPage}`);
      }
      if (this.sortColumn !== initialState.sortColumn) {
        hashArray.push(`sortcolumn=${this.sortColumn}`);
      }
      if (this.sortAscending !== initialState.sortAscending) {
        hashArray.push(`sortascending=${this.sortAscending}`);
      }
      if (this.searchQuery !== initialState.searchQuery) {
        hashArray.push(`search=${this.searchQuery}`);
      }
      return hashArray.length ? "?" + hashArray.join("&") : "";
    },
    _setHash(queryParams: QueryParams) {
      this.setCurrentPage(queryParams.page);
      this.setSortColumn(queryParams.sortcolumn);
      this.setSortAscending(queryParams.sortascending);
      this.setSearchQuery(queryParams.search);
    },
  },
});
